/*******************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only
* intended for use with Renesas products. No other uses are authorized. This
* software is owned by Renesas Electronics Corporation and is protected under
* all applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software
* and to discontinue the availability of this software. By using this software,
* you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2015 Renesas Electronics Corporation. All rights reserved.
*******************************************************************************/
/*******************************************************************************
* System Name  : RX111 initialization example
* File Name    : r_init_clock.c
* Version      : Ver 1.01
* Device       : R5F51115ADFM(RX111 Group)
* Abstract     : Program example of RX111 initialization
* Tool-Chain   : e2 studio (Version 2.2.0.11)
*              : C/C++ Compiler Package for RX Family (V2.01.00)
* OS           : not use
* H/W Platform : Renesas Starter Kit for RX111
* Description  : Initialize the clock.
* Limitation   : none
*******************************************************************************/
/*******************************************************************************
* History      : DD.MM.YYYY Version  Description
*              : 01.01.2014 1.00     First Release
*              : 02.03.2015 1.01     Change of the protect register settings
*******************************************************************************/

/*******************************************************************************
Includes <System Includes> , "Project Includes"
*******************************************************************************/
#include <stdint.h>
#include "..\src\iodefine.h"
#include "r_init_clock.h"
#include "r_delay.h"

/*******************************************************************************
Macro definitions
*******************************************************************************/
#define SUB_CLOCK_CYCLE  (1000000L/SUB_CLOCK_Hz)
#define LOCO_CLOCK_kHz    (4560L)

/* Time for one count of CMT0 is approximately 7018 ns
   when the count source is LOCO divided by 32 (max. of LOCO = 4.56 / 8 MHz) */
#define FOR_CMT0_TIME    (7018*8)

/*******************************************************************************
Private variables and functions
*******************************************************************************/
#if ((SEL_SUB == B_USE) || (SEL_RTC == B_USE))
    static void oscillation_subclk(void);
    static void cmt0_countstart(uint16_t cnt);
    static void cmt0_endcheck(void);
#if ((SEL_RTC == B_USE) && (SEL_SUB != B_USE))
    static void no_use_subclk_as_sysclk(void);
#endif
#if (SEL_RTC == B_USE)
    static void enable_RTC(void);
#endif
#endif

/*******************************************************************************
* Outline      : Clock initialization
* Header       : r_init_clock.h
* Function Name: R_INIT_Clock
* Description  : Initialize the clock.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void R_INIT_Clock(void)
{

    /* ---- Enable write protection ---- */
    /* PRCR - Protect Register
    b15:b8 PRKEY - PRC Key Code - A5h
                  (The write value should be A5h to permission writing PRCi bit)
    b2     PRC2 - Protect Bit 2 - Write enabled
    b1     PRC1 - Protect Bit 1 - Write enabled
    b0     PRC0 - Protect Bit 0 - Write enabled */
    SYSTEM.PRCR.WORD = 0xA507;

    /* ---- Set the sub-clock ---- */
#if ((SEL_SUB == B_USE) || (SEL_RTC == B_USE))
    CGC_oscillation_sub();
#else
    CGC_disable_subclk();
#endif

    /* ---- Set the main clock ---- */
#if ((SEL_MAIN == B_USE) || (SEL_PLL == B_USE))
    CGC_oscillation_main();
#endif

    /* ---- Set the PLL clock ---- */
#if (SEL_PLL == B_USE)
    CGC_oscillation_PLL();
#endif

    /* ---- Set the HOCO clock ---- */
#if (SEL_HOCO == B_USE)
    CGC_oscillation_HOCO();
#endif

#if ((REG_OPCCR == OPCM_MID) && (REG_SOPCCR == SOPCM_MID))
    while (0 != SYSTEM.OPCCR.BIT.OPCMTSF)
    {
        /* Check that operation power control mode transition is completed. */
    }

    /* ---- Set the operating power control mode ---- */
    /* OPCCR - Operating Power Control Register
    b7:b5   Reserved - The write value should be 0.
    b4      OPCMTSF  - Operating Power Control Mode Transition Status Flag
    b3      Reserved - The write value should be 0.
    b2:b0   OPCM     - Operating Power Control Mode Select - Middle-speed operating mode */
    SYSTEM.OPCCR.BYTE = REG_OPCCR;

    while (0 != SYSTEM.OPCCR.BIT.OPCMTSF)
    {
        /* Confirm that the operation power control mode transition completed. */
    }

#endif

    /* ---- Set the internal clock division ratio ---- */
    /* SCKCR - System Clock Control Register
    b31:b28 FCK      - FlashIF Clock(FCLK) Select - no division
    b27:b24 ICK      - System Clock (ICLK) Select - no division
    b23:b12 Reserved - The write value should be 0.
    b10:b8  PCLKB    - Peripheral Module Clock B(PCLKB) Select - no division
    b7:b4   Reserved - The write value should be 0.
    b3:b0   PCLKD    - Peripheral Module Clock D(PCLKD) Select - no division */
    SYSTEM.SCKCR.LONG = REG_SCKCR;
    while (REG_SCKCR != SYSTEM.SCKCR.LONG)
    {
         /* Confirm that the written value can be read correctly. */
    }

    /* ---- Set the internal clock source ---- */
    /* SCKCR3 - System Clock Control Register 3
    b10:b8  CKSEL    - Clock Source Select - Main clock oscillator is selected. */
    SYSTEM.SCKCR3.WORD = SEL_SYSCLK;
    while (SEL_SYSCLK != SYSTEM.SCKCR3.WORD)
    {
        /* Confirm that the written value can be read correctly. */
    }

#if ((REG_OPCCR == OPCM_LOW) && (REG_SOPCCR == SOPCM_LOW))

    /* ---- Operate the LOCO clock ---- */
    /* LOCOCR - Low-Speed On-Chip Oscillator Control Register
    b7:b1    Reserved   - The write value should be 0.
    b0       LCSTP      - the LOCO is operating. */
    SYSTEM.LOCOCR.BIT.LCSTP = 1;
    while (1 != SYSTEM.LOCOCR.BIT.LCSTP)
    {
        /* Loco stop check. */
    }

    R_DELAY_Us(1,SUB_CLOCK_Hz/1000);

    /* ---- Operate the main clock oscillator ---- */
    /* MOSCCR   - Main Clock Oscillator Control Register
    b7:b1    Reserved - The write value should be 0.
    b0       MOSTP    - Main Clock Oscillator Stop - Main clock oscillator is operating. */
    while (1 != SYSTEM.MOSCCR.BIT.MOSTP)
    {
        /* Main clock stop check. */
    }

    /* ---- Operate the PLL clock oscillator ---- */
    /* PLLCR2 - PLL Control Register 2
    b7:b1    Reserved - The write value should be 0.
    b0       PLLEN    - PLL Stop Control - PLL is operating. */
    while (1 != SYSTEM.PLLCR2.BIT.PLLEN)
    {
        /* PLL clock stop check. */
    }

    /* ---- Operate the HOCO clock ---- */
    /* HOCOCR - High-Speed On-Chip Oscillator Control Register
    b7:b1    Reserved   - The write value should be 0.
    b0       HCSTP      - the HOCO is operating. */
    while (1 != SYSTEM.HOCOCR.BIT.HCSTP)
    {
        /* HOCO stop check. */
    }

    while (0 != SYSTEM.SOPCCR.BIT.SOPCMTSF)
    {
        /* Check that operation power control mode transition is completed. */
    }

    /* ---- Set the operating power control mode ---- */
    /* SOPCCR - Operating Power Control Register
    b7:b5   Reserved - The write value should be 0.
    b4      SOPCMTSF - Sub Operating Power Control Mode Transition Status Flag
    b3      Reserved - The write value should be 0.
    b2:b0   SOPCM    - Operating Power Control Mode Select - Low-speed operating mode */
    SYSTEM.SOPCCR.BYTE = REG_SOPCCR;

    while (0 != SYSTEM.SOPCCR.BIT.SOPCMTSF)
    {
        /* Confirm that the operation power control mode transition completed. */
    }

#endif

    /* ---- Disable write protection ---- */
    /* PRCR - Protect Register
    b15:b8 PRKEY - PRC Key Code - A5h
                  (The write value should be A5h to permission writing PRCi bit)
    b1     PRC1 - Protect Bit 1 - Write disabled
    b0     PRC0 - Protect Bit 0 - Write disabled */
    SYSTEM.PRCR.WORD = 0xA500;

}

#if ((SEL_MAIN == B_USE) || (SEL_PLL == B_USE))
/*******************************************************************************
* Outline      : Configure main clock oscillation
* Header       : r_init_clock.h
* Function Name: CGC_oscillation_main
* Description  : Set the drive capability and the MOSCWTCR register,
*                then enable main clock oscillation.
*                Wait for the main clock oscillation stabilization
*                wait time by MOOVF bit.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void CGC_oscillation_main(void)
{

    /* ----  Set the main clock oscillator drive capability ---- */
    /* MOFCR - Main Clock Oscillator Forced Oscillation Control Register
    b7       Reserved - The write value should be 0.
    b6       MOSEL    - Main Clock Oscillator Switch - Resonator
    b5       MODRV21  - Main Clock Oscillator Drive Capability Switch
                      - 10 MHz to 20 MHz
    b4:b0    Reserved - The write value should be 0. */
    SYSTEM.MOFCR.BYTE = REG_MOFCR;

    /* ---- Set wait time until the main clock oscillator stabilizes ---- */
    /* MOSCWTCR - Main Clock Oscillator Wait Control Register
    b7:b5    Reserved - The write value should be 0.
    b4:b0    MSTS     - Main Clock Oscillator Waiting Time
                      - Wait time is 32768 cycles (approx. 8.192 ms). */
    SYSTEM.MOSCWTCR.BYTE = REG_MOSCWTCR;

    /* ---- Operate the main clock oscillator ---- */
    /* MOSCCR   - Main Clock Oscillator Control Register
    b7:b1    Reserved - The write value should be 0.
    b0       MOSTP    - Main Clock Oscillator Stop - Main clock oscillator is operating. */
    SYSTEM.MOSCCR.BYTE = 0x00;

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    while (1 != SYSTEM.OSCOVFSR.BIT.MOOVF)
    {
        /* Waits until MOOVF bit is set to 1. Main clock can be used after MOOVF bit is set to 1. */

    }

}
#endif

#if (SEL_PLL == B_USE)
/*******************************************************************************
* Outline      : Configure PLL clock oscillation
* Header       : r_init_clock.h
* Function Name: CGC_oscillation_PLL
* Description  : Set the PLL input frequency division ratio and 
*                frequency multiplication factor, set the PLLWTCR, 
*                then enable PLL clock oscillation.
*                Wait for the PLL clock oscillation stabilization time.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void CGC_oscillation_PLL(void)
{

    /* ---- Set the PLL division ratio and multiplication factor ---- */
    /* PLLCR - PLL Control Register
    b15:b13  Reserved - The write value should be 0.
    b12:b8   STC      - Frequency Multiplication Factor Select
                      - Frequency multiplication factor is multiply-by-6.
    b7:b2    Reserved - The write value should be 0.
    b1:b0    PLIDIV   - PLL Input Frequency Division Ratio Select
                      - PLL input division ratio is divide-by-2. */
    SYSTEM.PLLCR.WORD = REG_PLLCR;

    /* ---- Operate the PLL clock oscillator ---- */
    /* PLLCR2 - PLL Control Register 2
    b7:b1    Reserved - The write value should be 0.
    b0       PLLEN    - PLL Stop Control - PLL is operating. */
    SYSTEM.PLLCR2.BYTE = 0x00;

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    while (1 != SYSTEM.OSCOVFSR.BIT.PLOVF)
    {
        /* Waits until PLOVF bit is set to 1. Main clock can be used after PLOVF bit is set to 1. */

    }

}

#endif

#if (SEL_HOCO == B_USE)
/*******************************************************************************
* Outline      : Configure the HOCO clock oscillation
* Header       : r_init_clock.h
* Function Name: CGC_oscillation_HOCO
* Description  : Set the HOCOWTCR register then enable the HOCO.
*                Wait for the HOCO stabilization wait time by HCOVF bit.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void CGC_oscillation_HOCO(void)
{

    /* ---- Set wait time until the HOCO oscillator stabilizes ---- */
    /* HOCOWTCR - HOCO Wait Control Register
    b7:b5    Reserved - The write value should be 0.
    b4:b0    HSTS     - HOCO Wait Time Select
                      - Wait time is 266 cycles (approx. 66.5 us). */
    SYSTEM.HOCOWTCR.BYTE = REG_HOCOWTCR;

    /* ---- Operate the HOCO clock ---- */
    /* HOCOCR - High-Speed On-Chip Oscillator Control Register
    b7:b1    Reserved   - The write value should be 0.
    b0       HCSTP      - the HOCO is operating. */
    SYSTEM.HOCOCR.BYTE = 0x00;

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    while (1 != SYSTEM.OSCOVFSR.BIT.HCOVF)
    {
        /* Waits until HCOVF bit is set to 1. HOCO clock can be used after HCOVF bit is set to 1. */

    }

}

#endif

#if ((SEL_SUB == B_USE) || (SEL_RTC == B_USE))
/*******************************************************************************
* Outline      : Configure the Sub-clock oscillation
* Header       : r_init_clock.h
* Function Name: CGC_oscillation_sub
* Description  : Configure the setting when the sub-clock is used
*                as the system clock or RTC count source.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void CGC_oscillation_sub(void)
{

    /* ---- Setting of the sub-clock oscillation  ---- */
    oscillation_subclk();

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    cmt0_endcheck();

    /* ---- When using the RTC ---- */
#if (SEL_RTC == B_USE)
    enable_RTC();
#else
    /* RCR1 - RTC Control Register 1
    b2       PIE      - Periodic interrupt is disabled.
    b0       AIE      - Alarm Interrupt is disabled. */
    RTC.RCR1.BYTE &= 0xFA;
    /* RTC Alarm interrupt request is enabled */
    IEN(RTC,ALM) = 0;
    IR(RTC,ALM) = 0;
    /* RTC periodic interrupt request is enabled */
    IEN(RTC,PRD) = 0;
    IR(RTC,PRD) = 0;
#endif

    /* ---- Setting of the sub-clock do not use the system clock ---- */
#if (SEL_SUB == B_NOT_USE)
    no_use_subclk_as_sysclk();
#endif

}

#endif

#if ((SEL_SUB == B_USE) || (SEL_RTC == B_USE))
#else
/*******************************************************************************
* Outline      : Configure the Sub-clock stop
* Header       : r_init_clock.h
* Function Name: CGC_disable_subclk
* Description  : Configure the setting when the sub-clock is not used
*                as the system clock or RTC count source.
* Arguments    : none
* Return Value : none
*******************************************************************************/
void CGC_disable_subclk(void)
{

    uint8_t i;
    volatile uint8_t dummy;

    /* ---- Stop the sub-clock oscillator ---- */
    /* SOSCCR - Sub-Clock Oscillator Control Register
    b7:b1    Reserved - The write value should be 0.
    b0       SOSTP    - Sub-clock oscillator Stop - Sub-clock oscillator is stopped. */
    SYSTEM.SOSCCR.BYTE = 0x01;
    while (0x01 != SYSTEM.SOSCCR.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* RCR3 - RTC Control Register 3
    b0       RTCEN    - Sub-clock oscillator is stopped. */
    RTC.RCR3.BIT.RTCEN = 0;

    /* dummy read three times */
    for (i = 0; i < 3; i++)
    {
        dummy = RTC.RCR3.BIT.RTCEN;
    }

    while (0 != RTC.RCR3.BIT.RTCEN)
    {
        /* Confirm that the written */
    }

    /* RCR1 - RTC Control Register 1
    b2       PIE      - Periodic interrupt is disabled.
    b0       AIE      - Alarm Interrupt is disabled. */
    RTC.RCR1.BYTE &= 0xFA;
    /* RTC Alarm interrupt request is enabled */
    IEN(RTC,ALM) = 0;
    IR(RTC,ALM) = 0;
    /* RTC periodic interrupt request is enabled */
    IEN(RTC,PRD) = 0;
    IR(RTC,PRD) = 0;

}
#endif

#if ((SEL_SUB == B_USE) || (SEL_RTC == B_USE))
/*******************************************************************************
* Outline      : Sub-clock oscillation
* Header       : none
* Function Name: oscillation_subclk
* Description  : Configure the sub-clock oscillation setting.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void oscillation_subclk(void)
{

    uint8_t i;
    volatile uint8_t dummy;

    /* ---- Stop the Sub-clock oscillator ---- */
    /* SOSCCR - Sub-Clock Oscillator Control Register
    b0       SOSTP    - Sub-clock oscillator Stop - Sub-clock oscillator is stopped. */
    SYSTEM.SOSCCR.BYTE = 0x01;
    while (0x01 != SYSTEM.SOSCCR.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }
    /* RCR3 - RTC Control Register 3
    b0       RTCEN    - Sub-clock oscillator is stopped. */
    RTC.RCR3.BIT.RTCEN = 0;

    /* dummy read three times */
    for (i = 0; i < 3; i++)
    {
        dummy = RTC.RCR3.BIT.RTCEN;
    }

    while (0 != RTC.RCR3.BIT.RTCEN)
    {
        /* Confirm that the written */
    }

    /* ---- Wait for five sub-clock cycles ---- */
    R_DELAY_Us(SUB_CLOCK_CYCLE*5,LOCO_CLOCK_kHz/8);       /* Wait time is 5 sub-clock cycles (approx. 153 us). */

    /* ---- Setting of the sub-clock drive strength ---- */
    /* RCR3 - RTC Control Register 3
    b7:b4    Reserved - The write value should be 0.
    b3:b1    RTCDV    - Sub-clock Oscillator Drive Ability Control
                      - Drive ability for low CL
    b0       RTCEN    - Sub-clock Control
                      - Sub-clock oscillator is stopped. */
    RTC.RCR3.BYTE = REG_RCR3;

    /* dummy read three times */
    for (i = 0; i < 3; i++)
    {
        dummy = RTC.RCR3.BYTE;
    }

    while (REG_RCR3 != RTC.RCR3.BYTE)
    {
        /* Confirm that the written */
    }

    /* ---- Operate the Sub-clock oscillator ---- */
    /* SOSCCR - Sub-Clock Oscillator Control Register
    b0       SOSTP    - Sub-clock oscillator Stop - Sub-clock oscillator is running. */
    SYSTEM.SOSCCR.BYTE = 0x00;
    while (0x00 != SYSTEM.SOSCCR.BYTE)
    {
        /* Confirm that the written */
    }

    /* ---- Wait processing for the clock oscillation stabilization ---- */
    cmt0_countstart((uint16_t)(WAIT_TIME_FOR_SUB_OSCILLATION/FOR_CMT0_TIME+1));     /* oscillation stabilize (1.31s). */

}

#endif

#if (SEL_RTC == B_USE)
/*******************************************************************************
* Outline      : Initialization when using the RTC
* Header       : none
* Function Name: enable_RTC
* Description  : Configure the initialization when using the RTC.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void enable_RTC(void)
{

    uint8_t i;
    volatile uint8_t dummy;

    /* ---- Set RCR3 register ---- */
    /* RCR3 - RTC Control Register 3
    b0       RTCEN    - Sub-clock oscillator is running. */
    RTC.RCR3.BIT.RTCEN = 1;

    /* dummy read three times */
    for (i = 0; i < 3; i++)
    {
         dummy = RTC.RCR3.BIT.RTCEN;
    }

    while (1 != RTC.RCR3.BIT.RTCEN)
    {
         /* Confirm that the written value can be read correctly. */
    }

    /* ---- Wait for six the sub-clock cycles ---- */
    R_DELAY_Us(SUB_CLOCK_CYCLE*6,LOCO_CLOCK_kHz/8);    /* Wait time is 6 sub-clock cycles (approx. 183 us). */

    /* ---- Stop prescaler and counter ---- */
    /* RCR2 - RTC Control Register 2
    b0       START    - start - Prescaler is stopped. */
    RTC.RCR2.BIT.START = 0;
    while (0 != RTC.RCR2.BIT.START)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- Stop prescaler and counter ---- */
    /* RCR2 - RTC Control Register 2
    b7       CNTMD    - Count Mode Select - The calendar count mode */
    RTC.RCR2.BIT.CNTMD = SEL_CNTMD;
    while (SEL_CNTMD != RTC.RCR2.BIT.CNTMD)
    {
        /* Confirm that the written value can be read correctly. */
    }

    /* ---- RTC Software Reset ---- */
    /* RCR2 - RTC Control Register 2
    b1       RESET     - RTC Software Reset
                       - The prescaler and target registers are reset by RTC software reset.*/
    RTC.RCR2.BIT.RESET = 1;
    while (0 != RTC.RCR2.BIT.RESET)
    {
        /* Confirm that the "0" can be read correctly. */
    }

}

#endif

#if ((SEL_RTC == B_USE) && (SEL_SUB != B_USE))
/*******************************************************************************
* Outline      : Configure when not using the sub-clock as the system clock
* Header       : none
* Function Name: no_use_subclk_as_sysclk
* Description  : Set the SOSTP bit to 1 (sub-clock stops)
*                when the sub-clock is used only as the RTC count source.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void no_use_subclk_as_sysclk(void)
{

    /* ---- Stop the Sub-clock oscillator ---- */
    /* SOSCCR - Sub-Clock Oscillator Control Register
    b0       SOSTP    - Sub-clock oscillator Stop - Sub-clock oscillator is stopped. */
    SYSTEM.SOSCCR.BYTE = 0x01;
    while (0x01 != SYSTEM.SOSCCR.BYTE)
    {
        /* Confirm that the written value can be read correctly. */
    }

}

#endif

#if ((SEL_SUB == B_USE) || (SEL_RTC == B_USE))
/*******************************************************************************
* Outline      : Start Software wait with CMT0
* Header       : none
* Function Name: cmt0_countstart
* Description  : This function is to wait for the oscillation stabilization wait time.
* Arguments    : cnt : Wait for the time specified by an argument multiplied 
*                      by FOR_CMT0_TIME (us).
* Return Value : none
*******************************************************************************/
static void cmt0_countstart(uint16_t cnt) 
{

    /* ---- Exit the module stop state ---- */
    MSTP(CMT0) = 0;                         /* CMT0 and CMT1 exit the module stop state. */

    /* ---- Stop CMT0 counting ---- */
    /* CMSTR0 - Compare Match Timer Start Register 0
    b15:b2  Reserved  - The write value should be 0.
    b1      STR1      - Count Start 1 - CMT1.CMCNT count is stopped.
    b0      STR0      - Count Start 0 - CMT0.CMCNT count is stopped. */
    CMT.CMSTR0.BIT.STR0 = 0;                /* CMT0 count is stopped. */

    /* ---- Set the CMT0 count source and Enable the CMT0 compare match interrupt ---- */
    /* CMCR - Compare Match Timer Control Register
    b15:b8   Reserved  - The write value should be 0.
    b7       Reserved  - The write value should be 1.
    b6       CMIE      - Compare Match Interrupt Enable - Compare match interrupt enable.
    b5:b2    Reserved  - The write value should be 0.
    b1:b0    CKS       - Clock Select - PCLK/32 */
    CMT0.CMCR.WORD = 0x00C1;

    /* ---- Clear the CMT0 count ---- */
    /* CMCNT - Compare Match Timer Counter */
    CMT0.CMCNT = 0x0000;

    /* ---- Wait time set is other than 0 ? ---- */
    if (0 != cnt)
    {
        /* ---- Decrement the wait time ---- */
        cnt = (uint16_t)(cnt - 1);
    }

    /* ---- Set the CMT0 wait time ---- */
    /* CMCOR - Compare Match Timer Constant Register */
    CMT0.CMCOR = (unsigned short)cnt;

    /* ---- Clear the CMT0 interrupt request ---- */
    while (1 == IR(CMT0,CMI0))
    {
        IR(CMT0,CMI0) = 0;                  /* CMT0.CMI0 interrupt request is cleared. */
    }

    /* ---- Start CMT0 counting ---- */
    /* CMSTR0 - Compare Match Timer Start Register 0
    b0      STR0      - Count Start 0 - CMT0.CMCNT count is started. */
    CMT.CMSTR0.BIT.STR0 = 1;

}

#endif

#if ((SEL_SUB == B_USE) || (SEL_RTC == B_USE))
/*******************************************************************************
* Outline      : End check Software wait with CMT0
* Header       : none
* Function Name: cmt0_endcheck
* Description  : This function is to wait for the oscillation stabilization wait time.
* Arguments    : none
* Return Value : none
*******************************************************************************/
static void cmt0_endcheck(void)
{

    /* ---- CMI0 interrupt request generated? ---- */
    while (0 == IR(CMT0,CMI0))
    {

    }

    /* ---- Stop CMT0 counting ---- */
    /* CMSTR0 - Compare Match Timer Start Register 0
    b0      STR0      - Count Start 0 - CMT0.CMCNT count is stopped. */
    CMT.CMSTR0.BIT.STR0 = 0;

    /* ---- Initialize CMT0 ---- */
    /* CMCR - Compare Match Timer Control Register
    b15:b8   Reserved  - The write value should be 0.
    b7       Reserved  - The write value should be 1.
    b6       CMIE      - Compare Match Interrupt Enable - Compare match interrupt disable.
    b5:b2    Reserved  - The write value should be 0.
    b1:b0    CKS       - Clock Select - PCLK/8 */
    CMT0.CMCR.WORD = 0x0080;

    /* CMCNT - Compare Match Timer Counter */
    CMT0.CMCNT = 0x0000;

    /* CMCOR - Compare Match Timer Constant Register */
    CMT0.CMCOR = 0x0000;

    /* Clear IR flag */
    IR(CMT0,CMI0) = 0;

    /* ----Transfer the module stop state ---- */
    MSTP(CMT0) = 1;

}

#endif

/* End of File */

